@@ -7,7 +7,7 @@ import sys
 from itertools import count
 
 from . import opparse
-from .tags import Tag
+from .tags import Tag, tag as tag_factory
 from .utils import ABSENT
 
 
@@ -25,6 +25,7 @@ class InternedMC(type):
 
 
 class Element(metaclass=InternedMC):
+    """Represents a variable or some other atom."""
 
     _constructor_defaults = {
         "value": ABSENT,
@@ -96,6 +97,7 @@ class Element(metaclass=InternedMC):
             return set()
 
     def specialize(self, specializations):
+        """Replace $variables in the selector using a specializations dict."""
         spc = specializations.get(self.capture, ABSENT)
         if spc is ABSENT:
             return self
@@ -111,6 +113,7 @@ class Element(metaclass=InternedMC):
         return rval
 
     def encode(self):
+        """Return a string representation of the selector."""
         if self.name is None and self.capture is not None:
             name = f"${self.capture}"
             cap = ""
@@ -133,6 +136,7 @@ class Element(metaclass=InternedMC):
 
 
 class Call(metaclass=InternedMC):
+    """Represents a call in the call stack."""
 
     _constructor_defaults = {
         "children": (),
@@ -203,6 +207,7 @@ class Call(metaclass=InternedMC):
         return rval
 
     def specialize(self, specializations):
+        """Replace $variables in the selector using a specializations dict."""
         return self.clone(
             element=self.element and self.element.specialize(specializations),
             children=tuple(
@@ -214,6 +219,7 @@ class Call(metaclass=InternedMC):
         )
 
     def encode(self):
+        """Return a string representation of the selector."""
         name = self.element.encode()
         caps = []
         for cap in self.captures:
@@ -236,7 +242,7 @@ parser = opparse.Parser(
         {
             # r"\s*(?:\bas\b|>>|!+|\[\[|\]\]|[(){}\[\]>:,$=~])?\s*": "OPERATOR",
             r"\s*(?:\bas\b|>>|!+|\[\[|\]\]|[(){}\[\]>:,$=~])\s*|\s+": "OPERATOR",
-            r"[a-zA-Z_0-9#*.-]+": "WORD",
+            r"[a-zA-Z_0-9#@*.-]+": "WORD",
             r"'[^']*'": "STRING",
         }
     ),
@@ -259,6 +265,11 @@ parser = opparse.Parser(
 
 
 def _guarantee_call(parent, context, resolve=True):
+    """Always returns a Call instance.
+
+    If given an Element, return a Call with that Element as the function
+    to call. The focus and capture name are removed, if there are any.
+    """
     if isinstance(parent, Element):
         name = VSymbol(parent.name) if parent.name and resolve else parent.name
         parent = parent.clone(capture=None, name=name).without_focus()
@@ -508,7 +519,12 @@ def make_symbol(node, context):
 
 
 def dict_resolver(env):
+    """Resolve a symbol from a dictionary, e.g. the globals directory."""
+
     def resolve(x):
+        if x.startswith("@"):
+            return getattr(tag_factory, x[1:])
+
         start, *parts = x.split(".")
         if start in env:
             curr = env[start]
@@ -520,7 +536,7 @@ def dict_resolver(env):
         for part in parts:
             curr = getattr(curr, part)
 
-        return curr
+        return getattr(curr, "__ptera__", curr)
 
     return resolve
 
@@ -536,11 +552,11 @@ class VSymbol(VNode):
     def eval(self, env):
         x = self.value
 
-        if re.match(r"-?[0-9]+\.[0-9]*", x):
+        if re.fullmatch(r"-?[0-9]+\.[0-9]*", x):
             return float(x)
-        elif re.match(r"-?[0-9]+", x):
+        elif re.fullmatch(r"-?[0-9]+", x):
             return int(x)
-        elif re.match(r"'[^']*'", x):
+        elif re.fullmatch(r"'[^']*'", x):
             return x[1:-1]
         elif isinstance(env, dict):
             return dict_resolver(env)(x)
@@ -699,6 +715,17 @@ def _select(pattern, context="root"):
 
 
 def select(s, env=None, skip_modules=[]):
+    """Create a selector from a string.
+
+    Arguments:
+        s: The string to compile to a Selector, or a Selector to return
+            unchanged.
+        env: The environment to use to evaluate symbols in the selector.
+            If not given, the environment chosen is the parent scope.
+        skip_modules: Modules to skip when looking for an environment.
+            We will go up through the stack until we get to a scope that
+            is outside these modules.
+    """
     if not isinstance(s, str):
         return s
     if env is None:
