@@ -1,18 +1,24 @@
+"""Create objects out of functions."""
+
 import ast
 import builtins
 import inspect
+import re
 import tokenize
 from ast import NodeTransformer, NodeVisitor
 from copy import copy, deepcopy
 from textwrap import dedent
 from types import TracebackType
 
+from .tags import get_tags
 from .utils import ABSENT, keyword_decorator
 
 idx = 0
 
 
 class PteraNameError(NameError):
+    """The Ptera equivalent of a NameError, which gives more information."""
+
     def __init__(self, varname, function):
         self.varname = varname
         self.function = function
@@ -36,6 +42,7 @@ class PteraNameError(NameError):
 
 
 def name_error(varname, function, pop_frames=1):
+    """Raise a PteraNameError pointing to the right location."""
     fr = inspect.currentframe()
     for i in range(pop_frames + 1):
         if fr:
@@ -54,6 +61,10 @@ def name_error(varname, function, pop_frames=1):
 
 
 def readline_mock(src):
+    """Line reader for the given text.
+
+    This is meant to be used with Python's tokenizer.
+    """
     curr = -1
     src = bytes(src, encoding="utf8")
     lines = [line + b"\n" for line in src.split(b"\n")]
@@ -69,12 +80,28 @@ def readline_mock(src):
 
 
 def gensym():
+    """Generate a fresh symbol."""
     global idx
     idx += 1
     return f"_ptera_tmp_{idx}"
 
 
 class ExternalVariableCollector(NodeVisitor):
+    """Collect variables referred to but not defined in the given AST.
+
+    The attributes are filled after the object is created.
+
+    Attributes:
+        used: Set of used variable names (does not include the names of
+            inner functions).
+        assigned: Set of assigned variable names.
+        vardoc: Dict that maps variable names to matching comments.
+        provenance: Dict that maps variable names to "body" or "argument"
+            if they are defined as variables in the body or as function
+            arguments.
+        funcnames: Set of function names defined in the body.
+    """
+
     def __init__(self, comments, tree):
         self.used = set()
         self.assigned = set()
@@ -106,6 +133,12 @@ class ExternalVariableCollector(NodeVisitor):
 
 
 class PteraTransformer(NodeTransformer):
+    """Transform the AST of a function to instrument it with ptera.
+
+    The `result` field is set to the AST of the transformed function
+    after instantiation of the PteraTransformer.
+    """
+
     def __init__(self, tree, comments):
         super().__init__()
         evc = ExternalVariableCollector(comments, tree)
@@ -122,12 +155,27 @@ class PteraTransformer(NodeTransformer):
         self.result = self.visit_FunctionDef(tree, root=True)
 
     def fself(self):
+        """Create a Name for the self-function."""
         return ast.Name(id="__self__", ctx=ast.Load())
 
+    def _ann(self, ann):
+        if isinstance(ann, ast.Str) and ann.s.startswith("@"):
+            tags = re.split(r" *& *", ann.s)
+            ann = ast.Call(
+                func=ast.Name(id="__ptera_get_tags", ctx=ast.Load()),
+                args=[
+                    ast.Str(s=tag[1:]) for tag in tags if tag.startswith("@")
+                ],
+                keywords=[],
+            )
+        return ann
+
     def _absent(self):
+        """Create a Name that represents the lack of a value."""
         return ast.Name(id="__ptera_ABSENT", ctx=ast.Load())
 
     def make_interaction(self, target, ann, value, orig=None):
+        """Create code for setting the value of a variable."""
         if ann and isinstance(target, ast.Name):
             self.annotated[target.id] = ann
             self.linenos[target.id] = target.lineno
@@ -144,9 +192,11 @@ class PteraTransformer(NodeTransformer):
         elif isinstance(target, ast.Subscript) and isinstance(
             target.value, ast.Name
         ):
+            slc = target.slice
+            slc = slc.value if isinstance(target.slice, ast.Index) else slc
             value_args = [
                 ast.Constant(value=target.value.id),
-                deepcopy(target.slice.value),
+                deepcopy(slc),
                 ann_arg,
                 self.fself(),
                 value_arg,
@@ -195,7 +245,7 @@ class PteraTransformer(NodeTransformer):
                 target=ast.copy_location(
                     ast.Name(id=target.arg, ctx=ast.Store()), target
                 ),
-                ann=target.annotation,
+                ann=self._ann(target.annotation),
                 value=ast.copy_location(
                     ast.Name(id=target.arg, ctx=ast.Load()), target
                 ),
@@ -317,10 +367,10 @@ class PteraTransformer(NodeTransformer):
             x: int
 
         After::
-            x: int = ptera.interact('x', int)
+            x: int = _ptera_interact('x', int)
         """
         return self.make_interaction(
-            node.target, node.annotation, node.value, orig=node
+            node.target, self._ann(node.annotation), node.value, orig=node
         )
 
     def visit_Assign(self, node):
@@ -330,7 +380,7 @@ class PteraTransformer(NodeTransformer):
             x = y + z
 
         After::
-            x = ptera.interact('x', None, y + z)
+            x = _ptera_interact('x', None, y + z)
         """
         (target,) = node.targets
         if isinstance(target, ast.Tuple):
@@ -390,11 +440,23 @@ def transform(fn, interact):
     new_fn = compile(
         ast.Module(body=[new_tree], type_ignores=[]), filename, "exec"
     )
+
+    # We add a few extra variables in the existing namespace
     glb = fn.__globals__
     glb["__ptera_interact"] = interact
     glb["__ptera_ABSENT"] = ABSENT
+    glb["__ptera_get_tags"] = get_tags
+
+    fname = fn.__name__
+    save = glb.get(fname, None)
     exec(new_fn, glb, glb)
 
+    # Get the new function (populated with exec)
+    actual_fn = glb[fname]
+
+    # However, we don't want to change the existing mapping of fn
+    glb[fname] = save
+
     state = {
         k: override(
             eval(compile(ast.Expression(v), filename, "eval"), glb, glb),
@@ -403,8 +465,6 @@ def transform(fn, interact):
         for k, v in transformer.defaults.items()
     }
 
-    fname = fn.__name__
-    actual_fn = glb[fname]
     all_vars = transformer.used | transformer.assigned
 
     info = {
@@ -445,6 +505,13 @@ def transform(fn, interact):
 
 
 class Override:
+    """Represents a value with a priority.
+
+    Attributes:
+        value: The value.
+        priority: The priority.
+    """
+
     def __init__(self, value, priority=1):
         assert not isinstance(value, Override)
         self.value = value
@@ -452,6 +519,7 @@ class Override:
 
 
 def override(value, priority=1):
+    """Wraps a value to give it a priority."""
     if isinstance(value, Override):
         return value
     else:
@@ -459,6 +527,7 @@ def override(value, priority=1):
 
 
 def default(value, priority=-10):
+    """Wraps a value to give it a low priority (-10)."""
     return override(value, priority)
 
 
@@ -531,7 +600,7 @@ class Selfless:
 
 
 class ConflictError(Exception):
-    pass
+    """Represents an error due to different values having the same priority."""
 
 
 def choose(opts, name):
@@ -560,6 +629,7 @@ def choose(opts, name):
 
 
 def selfless_interact(sym, key, category, __self__, value):
+    """Simple interaction function for selfless objects."""
     from_state = __self__.get(sym)
     rval = choose([value, from_state], name=sym)
     if rval is ABSENT:
@@ -570,6 +640,7 @@ def selfless_interact(sym, key, category, __self__, value):
 
 @keyword_decorator
 def selfless(fn, **defaults):
+    """Create a selfless object."""
     new_fn, state = transform(fn, interact=selfless_interact)
     rval = Selfless(new_fn, state)
     if defaults:
